#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['pink', 'pink.models', 'pink.tasks']

package_data = \
{'': ['*'],
 'pink.models': ['jvrc_description/*',
                 'jvrc_description/meshes/*',
                 'jvrc_description/meshes/back/*',
                 'jvrc_description/rsdf/*',
                 'jvrc_description/rsdf/jvrc1/*',
                 'jvrc_description/urdf/*']}

install_requires = \
['numpy >=1.20.0', 'pin >=2.6.3', 'qpsolvers >=1.8.0']

extras_require = \
{'doc': ['sphinx']}

setup(name='pin-pink',
      version='0.0.2',
      description='Python inverse kinematics for your robot model based on Pinocchio.',
      author=None,
      author_email='Stéphane Caron <stephane.caron@normalesup.org>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.7',
     )
